#!/usr/bin/bash
## search-many-boards-imgur-2023.sh
## Run search against list of board shortnames.
## In Asagi, board shortnames are the same as their SQL table names.
##
## ==================================== ##
## Expects a working ~/.my.cnf for login.
##
## ==================================== ##
## AUTHOR: Ctrl-S
## CREATED: 2023-03-26
## MODIFIED: 2023-04-21
## TESTED: TODO
## ==================================== ##
# set -v # Print lines as they are run (bash option).
echo "#[${0##*/}]" "Starting" "$(date -Is)"
echo  "#[${0##*/}]" "Running as: $(whoami)@$(hostname) $(pwd)"


##=====< Configuration >=====##
table_names_file="tablenames.txt" ## Location of table list file.
sql_template_filepath="search.template.sql" # Location of SQL query template file.
mysql_login_file="$HOME/asagi.my.cnf" # e.g. "$HOME/.my.cnf"
db_name="asagi" # Name of mysql database to be used
job_dir="$PWD/tmp-work" # Base location to place results.
##=====< /Configuration >=====##

mkdir -vp "${job_dir?}"

job_name="imgur-search-2023.desuarchive.$(date -u +%Y-%m-%d_%H-%M-%S)" # Unique name for this run.
job_commonpath="${job_dir?}/${job_name?}" # Shared start of all output filepaths.


echo "#[${0##*/}]" "table names file is:"
ls -lahQZF $( realpath "${table_names_file?}" )


## =====< Loop over list >===== ##
echo "#[${0##*/}]" "Running searches using tables supplies in table names file: ${table_names_file?}"
c=0
(cat "${table_names_file?}"; ) | while read line; do
  let "c=c+1" # Simple line counter.
  echo "${table_names_file?}:${c}" "${line}" # i.e. filename.ext:ln LINE-TEXT
  
  ## =====< Handle list input >===== ##
  ## Skip header:
  pat_header='TABLE_NAME' 
  if [[ $line =~ $pat_header ]]; then echo "skip header"; continue; fi;

  ## Skip comments:
  pat_comment='^#'
  if [[ $line =~ $pat_comment ]]; then echo "skip comment"; continue; fi;    
  
  ## Skip non-alphanumeric:
  pat_nonalphanumeric='[^a-zA-Z0-9_-]'
  if [[ $line =~ $pat_nonalphanumeric ]]; then echo "skip non-alphanumeric"; continue; fi;

  ## Extract TABLE, from line:
  read -r table_name <<< "$line"; # Split on tab.
  echo  "table_name=$table_name"
  ## =====< /Handle list input >===== ##
  
  ## =====< Perform search task >===== ##
  echo "#[${0##*/}]" "Now handling table" "${table_name?}" "$(date -Is)"

  ## Create input SQL from template:
  query_filepath="${job_commonpath?}.query.${table_name?}.sql"
  echo "#[${0##*/}]" "query_filepath:" "${query_filepath?}"
  cp -v "${sql_template_filepath?}" "${query_filepath?}"
  sed -i "s/%%TABLE_NAME%%/${table_name?}/g" "${query_filepath?}"
  
  ## Run SQL query:
  mysql_params=( ## Array permits commenting.
    "--defaults-extra-file=${mysql_login_file?}"
    # --verbose # Echo SQL statements as they are read.
    --batch ## Force batch-mode formatting.
    "${db_name?}" 
  )
  echo "mysql_params=( $( builtin printf '%q ' "${mysql_params[@]}" ) )"
  mysql "${mysql_params[@]}" < "${query_filepath?}" \
    | tee >( md5sum > "${job_commonpath}.${table_name?}.tab.md5" ) \
    >  "${job_commonpath?}.${table_name?}.tab"

  echo "#[${0##*/}]" "Done handling table" "${table_name?}" "$(date -Is)"
  ## =====< /Perform search task >===== ##
done
## =====< /Loop over list >===== ##


## Bundle together so there's only one file to pass around:
tar --create --file "${job_name}.tar.gz" --no-recursion --gzip ${job_commonpath?}* "${table_names_file?}" "${sql_template_filepath?}" "${0}"


echo "#[${0##*/}]" "Finished" "$(date -Is)"
exit
